#!/usr/local/bin/python3
# /usr/bin/python3
# Set the path to your python3 above if needed

"""
GoAveragePlayer Simple heuristic-based Go player
Cmput 455 sample code
Written by Henry Du, based on Fuego AveragePlayer
"""

from board import GoBoard
from board_base import opponent, PASS
import gtp_connection
import random

class GoAveragePlayer():

    def __init__(self, board, generator):
        self.board = board
        self.generator = generator
        self.name = "AveragePlayer"
        self.version = 1.0

    def get_move(self, board, color):
        self.board = board
        moves_scores = self.generator.generate_moves(board, color)
        if not moves_scores:
            return PASS
        best_moves = []
        best_score = -float("inf")
        for move, score in moves_scores:
            if score > best_score:
                best_score = score
                best_moves.clear()
            if score >= best_score:
                best_moves.append(move)
        return random.choice(best_moves)


class AverageMoveGenerator():

    def __init__(self, board):
        self.board = board

    def generate_moves(self, board, color):
        """
        Performs 1-ply search and
        returns a list of (move, score) tuples
        """
        self.board = board
        empty_points = self.board.get_empty_points()
        moves_scores = []
        for point in empty_points:
            score = self.evaluate_move(point, color)
            # filter out eye-filling moves
            if self.board.is_eye(point, color):
                score = -float("inf")
            if score > -float("inf"):
                moves_scores.append((point, score))
        return moves_scores

    def evaluate_move(self, point, color):
        """
        Plays the move and returns the board evaluation as score
        """
        cboard = self.board.copy()
        if cboard.play_move(point, color):
            return self.evaluate(cboard)
        else:
            return -float("inf")

    def evaluate(self, board):
        """
        Evaluates the input board and returns its score
        """
        color = board.current_player
        opp_color = opponent(color)
        score = self.liberty_average(board, opp_color) - \
                self.liberty_average(board, color)
        return score

    def liberty_average(self, board, color):
        """
        Calculates the score of the input board w.r.t. color
        score = total_liberties(color) / number_of_blocks(color)
        """
        num_libs = num_blocks = 0
        met_points = []
        color_points = board.get_color_points(color)
        for point in color_points:
            if point not in met_points:
                liberty_count, block = board.get_block_liberty(point, color)
                met_points += block
                num_libs += liberty_count
                num_blocks += 1
        if num_blocks == 0:
            return 0
        return num_libs / num_blocks


def run():
    """
    Start the gtp connection and wait for commands.
    """
    size = 7
    board = GoBoard(size)
    average_move_generator = AverageMoveGenerator(board)
    average_player = GoAveragePlayer(board, average_move_generator)
    con = gtp_connection.GtpConnection(average_player, board)
    con.start_connection()


if __name__ == "__main__":
    run()

